package apis

import "context"

type MerchantApiInfo struct {
	IsForceUpdate      bool   `json:"is_force_update"`
	MerchantId         int64  `json:"merchant_id"`
	BaseApiId          int64  `json:"base_api_id"`
	MerchantChildApiId int64  `json:"merchant_child_api_id" description:"商户基础api id"`
	ReuseTime          int    `json:"reuse_time"`
	RangeNo            int    `json:"range_no"`
	OriginReuseTime    int    `json:"-"`
	RandomPercentage   int    `json:"random_percentage"`
	ApiTimeout         uint64 `json:"api_timeout" description:"商户接口超时时间"`
}

// 数据源原始数据
type ProviderRawData struct {
	Data      string `json:"data"`
	Timestamp int64  `json:"timestamp"`
}

type ManagementBaseApiParam struct {
	Required bool                     `json:"required" description:"是否必选"`
	Name     string                   `json:"name" description:"参数名字"`
	Mean     string                   `json:"mean" description:"参数含义"`
	Type     string                   `json:"type" description:"参数类型，整形、字符串等"`
	Example  string                   `json:"example" description:"参数示例"`
	Selected bool                     `json:"selected" description:"是否选择"`
	In       string                   `json:"in" description:"出参转换"`
	Child    []ManagementBaseApiParam `json:"child"`
}

type CommonResp struct {
	IsReuse  bool                `json:"is_reuse"`
	LReq     []ThirdpartLogWrite `json:"l_req"`
	ErrCode  int                 `json:"err_code"`
	ErrMsg   string              `json:"err_msg"`
	SySource string              `json:"sy_source"`
}

type RawData struct {
	Data      string `json:"data"`
	Timestamp int64  `json:"timestamp"`
}

type CommonReq struct {
	Data interface{} `json:"data"`
	Code string      `json:"code"`
	MerchantApiInfo
	ProviderMap  map[string]MerchantProviderLimitInfo `json:"provider_map"`
	ProviderGroup [][]string `json:"provider_group"`
	NeedVehicleInfo bool
}

type CommonReply struct {
	Data    interface{}         `json:"data"`
	IsReuse bool                `json:"is_reuse"`
	LReq    []ThirdpartLogWrite `json:"l_req"`
	ErrCode int                 `json:"err_code"`
	ErrMsg  string              `json:"err_msg"`
}

type ThirdpartRequest struct {
	Ctx             context.Context
	Params          map[string]string
	ProviderApiCode string
	ProviderMap     map[string]MerchantProviderLimitInfo
	RangeNo         int
	Req             *CommonReq
	Reply           *CommonReply
	// center中填充
	LReq         *ThirdpartLogWrite
	ProviderInfo MerchantProviderLimitInfo
	TryAgain bool
	Times int
}
