package cdbd

import (
	"fmt"
	"gd_vehicle/apis"
	"gd_vehicle/consts"
	"gd_vehicle/errors"
	"gd_vehicle/thirdparty"
	dutils "gd_vehicle/utils"
	"github.com/astaxie/beego/orm"
	"strconv"
	"time"

	"gd_vehicle/common.in/utils"
	"go.uber.org/zap"

	"github.com/tidwall/gjson"
)

func GetCdbdTwoElementVerifyData(rdata string, respMap map[string]string) {
	isMatch := gjson.Get(rdata, "match_result").String()
	respMap["is_match"] = isMatch
	if isMatch == "1" {
		respMap["state"] = gjson.Get(rdata, "car_detail.car_state").String()
		//respMap[""] = gjson.Get(rdata,"car_detail.reg_date").String()
		respMap["brand_name"] = gjson.Get(rdata, "car_detail.model_name").String()
		respMap["model_no"] = gjson.Get(rdata, "car_detail.model_type").String()
		respMap["vehicle_type"] = gjson.Get(rdata, "car_detail.vehicle_type").String()
		respMap["vin"] = gjson.Get(rdata, "car_detail.vin").String()
		respMap["initial_registration_date"] = gjson.Get(rdata, "car_detail.initial_reg_date").String()
	}
}

func getCdbdKey(req *apis.ThirdpartRequest) string {
	key := consts.CdbdTwoElementVerifyPre + req.Params["id_card"] + req.Params["plate_num"]
	return key
}

func paramCdbdData(ret string) (reply string, err error) {
	code := gjson.Get(ret, "code").Int()
	if code == 200 {
		result := gjson.Get(ret, "result").String()
		decryptMsg, err := thirdparty.SM4ECBBase64Decrypt(result)
		if err != nil {
			fmt.Println("SM4ECBBase64Decrypt error:", err)
		}
		return decryptMsg, nil
	} else if code == 60000 {
		return "", errors.NoRecord
	} else {
		return "", errors.VendorError
	}
}

func TwoElementVerifyLocal(req *apis.ThirdpartRequest) (reply string, err error) {
	reply, err = dutils.GetDataByRedis(getCdbdKey(req), int64(req.Req.ReuseTime), paramCdbdData)
	return reply, err
}

func StoreDataToMysql(owner string, data map[string]string, respData string) {
	authorization := data["authorization"]
	idCard := data["id_card"]
	plateNum := data["plate_num"]
	cdbdData := apis.CdbdData{}
	cdbdData.PlateNo = plateNum
	cdbdData.Name = owner
	cdbdData.IdCard = idCard
	cdbdData.Authorization = authorization
	cdbdData.Timestamp = time.Now().Unix()
	cdbdData.Data = respData
	o := orm.NewOrm()
	o.Insert(cdbdData)
}

func TwoElementVerify(req *apis.ThirdpartRequest) (reply string, err error) {
	//req.LReq.Search = req.Params["id_card"]
	owner := req.Params["owner"]
	delete(req.Params, "owner")

	ret, err := thirdparty.CdBdClientPost(req.ProviderInfo.ThirdpartHost, req.ProviderInfo.ThirdpartApiRouter, req.Params, req.ProviderInfo.Timeout)
	if err != nil {
		l.Error("func",
			zap.String("call", "CdBdClientPost"),
			zap.String("args", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", "三方错误,请求失败", false)
		return "", errors.VendorError
	} else {
		req.LReq.ResponseParams = string(ret)
		reply, err = paramCdbdData(string(ret))
		message := gjson.GetBytes(ret, "message").String()
		if err == nil {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 0, "200", message, true)
			dutils.StoreDataToRedis(getCdbdKey(req), string(ret))
			StoreDataToMysql(owner, req.Params, req.LReq.ResponseParams)
		} else if err == errors.NoRecord {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 1100, "60000", message, false)
			dutils.StoreDataToRedis(getCdbdKey(req), string(ret))
		} else {
			code := gjson.GetBytes(ret, "code").Int()
			l.Error("func",
				zap.String("call", "CdBdClientPost"),
				zap.String("args", utils.MarshalJsonString(req.Params)),
				zap.String("error", err.Error()))
			dutils.SetAccessLogReqCodeMsgState(req.LReq, int(code), strconv.Itoa(int(code)), message, false)
		}

		return reply, err

	}
}
