package dybd

import (
	"gd_vehicle/apis"
	"gd_vehicle/consts"
	"gd_vehicle/errors"
	"gd_vehicle/thirdparty"
	dutils "gd_vehicle/utils"
	"strconv"
	"time"

	"github.com/astaxie/beego/orm"

	"gd_vehicle/common.in/utils"
	"go.uber.org/zap"

	"github.com/tidwall/gjson"
)

func GetDybdTwoElementVerifyData(rdata string, respMap map[string]string) {
	arr := gjson.Parse(rdata).Array()
	for _, v := range arr {
		respMap["csrq"] = v.Get("csrq").String()
		//respMap[""] = gjson.Get(rdata,"car_detail.reg_date").String()
		respMap["xb"] = v.Get("xb").String()
		respMap["xkzy"] = v.Get("xkzy").String()
		respMap["xwlb"] = v.Get("xwlb").String()
		respMap["xwsydw"] = v.Get( "xwsydw").String()
		respMap["xm"] = v.Get( "xm").String()
		respMap["hxwrq"] = v.Get( "hxwrq").String()
	}
}

func getDybdTwoElementVerifyKey(req *apis.ThirdpartRequest) string {
	key := consts.DybdTwoElementVerifyPre + req.Params["xwzsbh"] + req.Params["xm"]
	return key
}

func paramDybdData(ret string) (reply string, err error) {
	code := gjson.Get(ret, "code").Int()
	if code == 200 {
		result := gjson.Get(ret, "data").String()
		arr := gjson.Parse(result).Array()
		if len(arr) == 0 {
			return result, errors.NoRecord
		} else {
			return result, nil
		}
	} else {
		return "", errors.VendorError
	}
}

func TwoElementVerifyLocal(req *apis.ThirdpartRequest) (reply string, err error) {
	reply, err = dutils.GetDataByRedis(getDybdTwoElementVerifyKey(req), int64(req.Req.ReuseTime), paramDybdData)
	return reply, err
}

func StoreDataToMysql(owner string, data map[string]string, respData string) {
	authorization := data["authorization"]
	idCard := data["id_card"]
	plateNum := data["plate_num"]
	cdbdData := apis.CdbdData{}
	cdbdData.PlateNo = plateNum
	cdbdData.Name = owner
	cdbdData.IdCard = idCard
	cdbdData.Authorization = authorization
	cdbdData.Timestamp = time.Now().Unix()
	cdbdData.Data = respData
	o := orm.NewOrm()
	o.Insert(cdbdData)
}

func TwoElementVerify(req *apis.ThirdpartRequest) (reply string, err error) {
	//req.LReq.Search = req.Params["xwzsbh"]
	//owner := req.Params["owner"]

	ret, err := thirdparty.DyBdClientPost(req.ProviderInfo.ThirdpartHost+"/"+req.ProviderInfo.ThirdpartApiRouter, req.Params, req.ProviderInfo.Timeout)
	if err != nil {
		l.Error("func",
			zap.String("call", "DyBdClientPost"),
			zap.String("args", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", "三方错误,请求失败", false)
		return "", errors.VendorError
	} else {
		req.LReq.ResponseParams = string(ret)
		reply, err = paramDybdData(string(ret))
		message := gjson.GetBytes(ret, "msg").String()
		if err == nil {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 0, "200", message, true)
			dutils.StoreDataToRedis(getDybdTwoElementVerifyKey(req), string(ret))
			//StoreDataToMysql(owner, req.Params, req.LReq.ResponseParams)
		} else if err == errors.NoRecord {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 1100, "gd_1101", message, false)
			dutils.StoreDataToRedis(getDybdTwoElementVerifyKey(req), string(ret))
		} else {
			code := gjson.GetBytes(ret, "code").Int()
			l.Error("func",
				zap.String("call", "DyBdClientPost"),
				zap.String("args", utils.MarshalJsonString(req.Params)),
				zap.String("error", err.Error()))
			dutils.SetAccessLogReqCodeMsgState(req.LReq, int(code), strconv.Itoa(int(code)), message, false)
		}

		return reply, err

	}
}
